#!/usr/bin/env python3

import configparser

# This class is used to remove spaces while writing an ini file to disk so that we can results of var=value instead
# of var = value
class INISpaceRemover:
    output_file = None

    def __init__( self, new_output_file ):
        self.output_file = new_output_file

    def write( self, what ):
        self.output_file.write( what.replace( " = ", "=" ) )


class INIParser(object):
    # We specifically refuse to allow characters that cannot be parsed back in since the base extension doesn't
    # do a sanity check on set or write
    forbidden_characters = ["@", "%", "^", "`", ";", "#", "[", "]", "/", "\\", "//", "+", "="]

    def __init__(self,ini_path):
        self.ini_path = ini_path
        print ("\niniparser: Path passed to init: " + ini_path)

        # Initialize a new configparser object immediately
        self.config = configparser.ConfigParser()
        assert isinstance(self.config, object)
        # This option tells the parser to respect case of both var and value names
        # The normal behavior is to lowercase everything which is not wanted
        self.config.optionxform = str
        print("iniparser: Initializing ConfigParser for: " + str(self.ini_path))

    # Create and return parser object
    def create(self) -> object:
        print("iniparser: Creating new ini file in memory: " + self.ini_path)

        return self.config

    # Read existing file into parser object
    def read(self,ini_path=''):
        if(ini_path):
            self.ini_path = ini_path

        try:
            self.config.read(self.ini_path)

        except Exception as e:
            print("iniparser: Exception: " + str(e))
            return False

        return True

    # Return parser object
    def get_config_object(self):
        return self.config

    # Add a new section to the parser object
    def add_section(self,section) -> bool:
        print("iniparser: Adding section: " + str(section))

        if (set(list(section)).intersection(self.forbidden_characters)) or not section:
            print("iniparser: Section: " + section + " contains illegal character: " +
                str(set(list(section)).intersection(self.forbidden_characters)))
            return False

        try:
            self.config.add_section(section)

        except Exception as e:
            print("iniparser: Exception: " + str(e))
            return False

        return True

    # Get a list of section names from the parser object
    def get_sections(self) -> list:
        return self.config.sections()

    # Get a variable from the parser object
    def get(self,section,variable):
        result = self.config.get(section, variable)
        print("iniparser: Getting variable: " + str(variable) + " : " + str(result))

        return result

    # Get a boolean variable from the parser object
    def getboolean(self,section,variable):
        result = self.config.getboolean(section, variable)
        print("iniparser: Getting boolean: " + str(variable) + " : " + str(result))

        return result

    # Set a variable in the parser object
    def set(self,section,variable,value):
        print("iniparser: Setting variable: '" + str(variable) + "' : '" + str(value) + "'")

        if ((type(variable) != str) or (type(value) != str)):
            print ("iniparser: Cannot add variable or/with value that is not a string!")
            return False

        if not variable or (set(list(variable)).intersection(self.forbidden_characters)):
            print("iniparser: Variable: " + variable + " contains illegal character: " +
                str(set(list(variable)).intersection(self.forbidden_characters)))
            return False

        if (set(list(value)).intersection(self.forbidden_characters)):
            print("iniparser: Variable: " + variable + " Value: " + value + " contains illegal character: " +
                str(set(list(value)).intersection(self.forbidden_characters)))
            return False

        try:
            self.config.set(section,variable,value)

        except Exception as e:
            print("iniparser: Exception: " + str(e))
            return False

        return True

    # Remove an entire section in the parser object
    def remove_section(self,section):
        print("iniparser: Removing Section: " + str(section))
        try:
            self.config.remove_section(section)

        except Exception as e:
            print("iniparser: Exception: " + str(e))
            return False

        return True

    # Print the entire parser object contents in a nice structured view
    def print_all(self):
        print ("")
        for each_section in self.config.sections():
            print (each_section)
            for (each_key, each_val) in self.config.items(each_section):
                print ("\t" + str(each_key) + " : " + str(each_val))

    # Write the parser object to a file on disk
    def write(self):
        print("\niniparser: Writing ini file: " + self.ini_path)
        self.print_all()

        try:
            with open(self.ini_path, 'w') as inifile:

                self.config.write( INISpaceRemover(inifile) )

        except Exception as e:
            print ("iniparser: Exception: " + str(e))
            return False

        return True
