#!/usr/bin/env python3

import sys
import os
import errno
import tempfile
import unittest
import configparser

# if script's parent directory isn't in sys.path, then add it
if ('/'.join(sys.path[0].split("/")[:-1]) not in sys.path):
    sys.path.append('/'.join(sys.path[0].split("/")[:-1]))

from apcommon import iniparser


class INIParserTests(unittest.TestCase):
    test_sections = ["Test", "Test2", "Test3", "Test4"]
    malformed_sections = ["%", "@#%$!", "^", "`", ";", "#", "[", "]", "/", "\\", "//", "+", "="]

    test_variables = {"var1": "1", "var2": "2", "var3": "3"}
    duplicate_variables = {"var1": "1", "var2": "2"}
    malformed_variables = {"%": "%", 1: 1, 2: 2, "var1": 1, "var2": 2, "#": "#", "+": "+",
                                "/": "/",  "//": "//",";": ";","=": "=", "[": "[", "]": "]" , "\\": "\\", "^": "^",
                                "`": "`"}

    def setUp(self):
        self.longMessage = True

        self.tmpfile = self.create_tmp_file()
        self.ini_class = iniparser.INIParser(self.tmpfile.name)
        self.config = self.ini_class.create()

        self.assertIsInstance(self.config, configparser.ConfigParser,
                              msg="Configparser object was not created by INIParser")

    def tearDown(self):
        self.silent_remove(self.tmpfile)

    @staticmethod
    def silent_remove(tmpfile: tempfile.NamedTemporaryFile):
        print("\nDelete: " + tmpfile.name)
        tmpfile.close()

        try:
            os.remove(tmpfile.name)
        except OSError as e:
            if e.errno != errno.ENOENT:  # errno.ENOENT = no such file or directory
                raise  # re-raise exception if a different error occured

    @staticmethod
    def create_tmp_file() -> tempfile.NamedTemporaryFile:
        tmpfile = tempfile.NamedTemporaryFile(delete=False)
        assert isinstance(tmpfile, object)
        print("\nTemporary Filename: " + tmpfile.name)

        return tmpfile

    def test_create_sections(self):
        print ("\nTesting Create New Sections in ini File...")

        # Test adding sections with weird characters to ini
        for section in self.test_sections:
            result = self.ini_class.add_section(section)
            self.assertTrue(result, msg="Failed to add section: " + section)

        added_sections = self.ini_class.get_sections()
        self.assertListEqual(self.test_sections,added_sections,
                             msg="Sections created do not match sections returned")

        # Test adding duplicate sections, they should always fail
        for section in self.test_sections:
            result = self.ini_class.add_section(section)
            self.assertFalse(result, msg="Duplicate section added: " + section)

    def test_remove_sections(self):
        print ("\nTesting Removing Sections in ini File...")

        # Test removing sections
        for section in self.test_sections:
            result = self.ini_class.add_section(section)
            self.assertTrue(result, msg="Failed to add section: " + section)

        for section in self.test_sections:
            result = self.ini_class.remove_section(section)
            self.assertTrue(result, msg="Failed to remove section: " + section)

        remaining_sections = self.ini_class.get_sections()

        empty_list = []
        print ("Remaining Sections: " + ','.join(remaining_sections))
        self.assertListEqual(remaining_sections,empty_list, msg="Not all sections were removed successfully")

    def test_add_malformed_sections(self):
        print ("\nTesting Removing Malformed Sections in ini File...")

        # Test removing malformed sections
        for section in self.malformed_sections:
            result = self.ini_class.add_section(section)
            self.assertFalse(result, msg="Failed to add section: " + section)

        # for section in self.malformed_sections:
        #     result = self.ini_class.remove_section(section)
        #     self.assertTrue(result, msg="Failed to remove section: " + section)

        remaining_sections = self.ini_class.get_sections()

        empty_list = []
        print ("Remaining Sections: " + ','.join(remaining_sections))
        self.assertListEqual(remaining_sections,empty_list, msg="Not all sections were removed successfully")

    def test_set_variables(self):
        print ("\nTesting Set Variables in Sections in ini File...")

        # Test setting variables and then resetting them with duplicate variables
        for section in self.test_sections:
            result = self.ini_class.add_section(section)
            self.assertTrue(result, msg="Failed to add section: " + section)

            for variable, value in sorted(self.test_variables.items()):
                result = self.ini_class.set(section,variable,value)
                self.assertTrue(result, msg="Failed to set variable: " + variable)

            for variable, value in sorted(self.duplicate_variables.items()):
                result = self.ini_class.set(section,variable,value)
                self.assertTrue(result, msg="Duplicate variable not added: " + variable)

    def test_set_malformed_variables(self):
        print ("\nTesting Set Malformed Variables in Sections in ini File...")

        # Test adding variables with weird characters to ini
        for section in self.test_sections:
            result = self.ini_class.add_section(section)
            self.assertTrue(result, msg="Failed to add section: " + section)

            for variable, value in self.malformed_variables.items():
                result = self.ini_class.set(section,variable,value)
                self.assertFalse(result, msg="Failed to catch setting malformed variable: " + str(variable))

    def test_get_variables(self):
        print("\nTesting Getting Variables in ini File...")

        # Add sections and variables we can test on
        for section in self.test_sections:
            result = self.ini_class.add_section(section)
            self.assertTrue(result, msg="Failed to add section: " + section)

            for variable, value in sorted(self.test_variables.items()):
                result = self.ini_class.set(section,variable,value)
                self.assertTrue(result, msg="Failed to set variable: " + variable)

        # loop through all sections/variables and see if they're all same values as expected
        for section in self.test_sections:
            for variable, value in sorted(self.test_variables.items()):
                result = self.ini_class.get(section, variable)
                self.assertEqual(result, value, msg="Failed to get variable: " + variable)

    def test_write_ini_file(self):
        print ("\nTesting Writing ini File...")

        for section in self.test_sections:
            result = self.ini_class.add_section(section)
            self.assertTrue(result, msg="Failed to add section: " + section)

            for variable, value in sorted(self.test_variables.items()):
                result = self.ini_class.set(section,variable,value)
                self.assertTrue(result, msg="Failed to set variable: " + variable)

        result = self.ini_class.write()
        self.assertTrue(result, msg="Failed to write ini file: " + self.tmpfile.name)

        # Write them to file and read them in and see what happens to the parser
        # see also what happens to the file

        ini_class2 = iniparser.INIParser(self.tmpfile.name)
        ini_class2.read(self.tmpfile.name)
        config2 = ini_class2.get_config_object()
        self.assertIsInstance(config2, configparser.ConfigParser,
                              msg="Configparser object was not created by INIParser")

        self.assertTrue(ini_class2.read(self.tmpfile.name), msg="Failed to read ini file: " + self.tmpfile.name)

        print ("\nTesting Comparing Sections From Written ini File vs What Was Written")
        sections = self.ini_class.get_sections()
        sections2 = ini_class2.get_sections()
        self.assertListEqual(sections,sections2, msg="Sections in initial ini file do not match ini re-read from disk")

        for section in sections2:
            self.assertEqual(dict(config2.items(section)),self.test_variables, msg="Variables/Values in ini file on "
                                                                                   "disk do not match ini file written")

        ##############
        # Add a test for get_boolean
        ##############

if __name__ == '__main__':
    INIParserTests.main()

